function param = GetDefaultParameters()

% Parameters initialization
param = [];


%       STANDARD VALUES
%       param.im.ratio : double
param.im.ratio = 1;
%       param.im.method = {'piecewise', 'linear', 'spline-3',
%                           'spline-5', 'fourier'}
param.im.method = 'piecewise';

%       param.im.correlation : 0 or 1
param.im.correlation = 0;
%       param.im.correlation_filter : correlation filter
param.im.correlation_filter = fspecial('gaussian', [7 7], 3);
%       param.im.median : 0 or 1
param.im.median = 0;
%       param.im.median_size : [m n] size of the median window
param.im.median_size = [10 10];
%       param.im.wiener : 0 or 1
param.im.wiener = 0;
%       param.im.wiener_size : [m n] size of the wiener window
param.im.wiener_size = [5 5];

%       param.grad.ratio    : double (typically, 1...)
param.grad.ratio = 1;
%       param.grad.method   = {'correlation', 'fourier'}
param.grad.method = 'correlation';
%       param.grad.operator : local correlation filter ?
param.grad.operator= 0.125 * ...
    [-1, 0, 1; ...
    -2, 0, 2; ...
    -1, 0, 1];
param.grad.operator(:,:,2)  = 0.125 * ...
    [-1, -2, -1; ...
    0,  0,  0; ...
    1,  2,  1];

% param.grad.operator= 1 * ...
%     [0, 0, 0; ...
%     -1, 0, 1; ...
%     0, 0, 0];
% param.grad.operator(:,:,2)  = 1 * ...
%     [0, -1, 0; ...
%     0,  0,  0; ...
%     0,  1,  0];
% param.grad.operator= 1 * ...
%     [0, 0, 0; ...
%     0, -1, 1; ...
%     0, 0, 0];
% param.grad.operator(:,:,2)  = 1 * ...
%     [0, 0, 0; ...
%     0,  -1,  0; ...
%     0,  1,  0];

%       param.grad.processing_method : {'none', 'correlation',
%               'median'}
param.grad.processing_method = 'none';
%       param.grad.median_size : [h w] size of the median window
param.grad.median_size = [10 10];
%       param.grad.blur : correlation filter ?
%       See fspecial for interesting examples.
param.grad.blur = fspecial('gaussian', [7 7], 3);
%       param.grad.pre_rotation : double (degree)
%       param.grad.post_rotation: double (degree)
%           Use those two parameters to rotate the vector field :
%               using 90 will typically end up with ~contour lines
param.grad.pre_rotation = 0;
param.grad.post_rotation = 0;

%       param.grad.interpolation_method : {'piecewise', 'bilinear'}
param.grad.interpolation_method = 'piecewise';


%       param.mask.start.initialize.type = {'ones', 'topological',
%           'set'}
param.mask.start.initialize.type = 'ones';
%       param.mask.start.initialize.set_value : matrix
param.mask.start.initialize.set_value = [];

%       param.mask.start.type = {'multiplicative', 'additive'}
%           N.B. : binary is a sub-type of multiplicative
param.mask.start.type = 'multiplicative';
%       param.mask.start.method = {'random', 'left-top'}
%           Indicates which point we should chose among the argmax of
%           mask.start .
param.mask.start.method = 'random';
%       param.mask.start.filter : local filter matrix, applied to
%           mask.start centered around x_0
param.mask.start.filter = fspecial('disk', 20) == 0;%zeros(41, 41);
%param.mask.start.filter = -0.15 * ones(21, 21);
%       param.mask.start.thresh : double
%           N.B. : typically, between 0 and 1
param.mask.start.thresh = 0.5;

%       param.computeline.method = {'cauchy', 'explicit-bilinear',
%           'explicit-piecewise', 'runge-kutta-23', 'runge-kutta-45'}
param.computeline.method = 'explicit-piecewise';
%       param.computeline.ode_options
%           Useful for controlling error when using runge-kutta
param.computeline.ode_options = odeset('MaxStep', 0.05);

%       param.computeline.time_step : double
%           N.B. : typically, around 0.1
param.computeline.time_step = 0.01;
%       param.computeline.direction = {'both', 'forward', 'backward'}
param.computeline.direction = 'both';

%       param.computeline.stopping.time.type = {'time', 'arc-length'}
%           N.B. : this is the exploration time in each direction
param.computeline.stopping.time.type = 'arc-length';
%       param.computeline.stopping.time.val  : double
%           N.B. : choose a ridiculously large value for non-stopping
%           lines
param.computeline.stopping.time.val = 2000;

%       param.computeline.stopping.grad_norm_threshold : double
%           Stops to draw line if the gradient norm at the current point
%           is below a fixed threshold.
param.computeline.stopping.grad_norm_threshold = 0.1;

%       param.mask.continue.type = {'multiplicative', 'additive'}
%           N.B. : binary is a sub-type of multiplicative
param.mask.continue.type = 'multiplicative';
%       param.mask.continue.filter : correlation matrix that will be
%           applied to mask.continue around every line point.
param.mask.continue.filter = 1-10*fspecial('gaussian', [11 11], 5); %0.9 * ones(11,11);
%       param.mask.continue.thresh : double
%           N.B. : Typically, between 0 and 1
param.mask.continue.thresh = 0.5;

%       param.computeline.max_line_size :
%           Typically around 500, the maximum number of points a line
%           may contain in each direction.
param.computeline.max_line_size = 2000;

%       param.n_lines : int
%           The max number of gradient lines we draw.
param.n_lines = 10;

%       param.display.markertype : string
%           See scatter doc for list of values, such as 'none', '+', ...
param.display.markertype = '.';
%       param.display.line : 0 or 1
%           Should we plot a line between the points ?
param.display.line = 1;
%       param.display.linespec : string
%           Line parameters : see plot doc.
param.display.linespec = '-b';
%       param.display.linewidth : int
%           Line parameters : see plot doc.
param.display.linewidth = 1;
%       param.display.line_alpha : 0 to 1
%           transparency of the line
param.display.line_alpha = 1;

%       param.display.quiver : 0 or 1
%           Should we plot small arrows ?
param.display.quiver = 0;
%       param.display.scatter : 0 or 1
%           Should we plot points ?
param.display.scatter = 1;
%       param.display.area_param = {'const', 'grad', 'arc-length', 'time'}
%           Which parameters controls the area of the markers
param.display.area_param = 'grad';
%       param.display.area_fun : param -> area function handle
%           The function that maps the area_param to an area, in pix^2
param.display.area_fun = @(x) (max(10*sqrt(norm(x)), 1));
%       param.display.color_param = {'const', 'grad', 'arc-length', 'time'}
%           Which parameters controls the color of the markers
param.display.color_param = 'arc-length';
%       param.display.color_fun : param -> [R;G;B] function handle
%           The function that maps the color_param to a RGB color (column
%           vector)
param.display.color_fun = @(x) 0.5*(sawtooth(x*2*pi/30) + 1)*[1;0.3;-1] + [0;0;1];

%       param.background.image = { 'none',
%           'original image', 
%           'interpolated image', 'processed image',
%           'start mask',
%           'gradient', 'null gradient', 'below-threshold gradient'}
param.background.image = 'original image';


end